local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

function Capitalist:StartPost()
	self:DebugEntry("StartPost")

	if (not self.InPostIteration) then
		ClearCursor()
		self.productsToPost = self:GetProductsInInventory()
	
		self.auctionUI.lastScanLabel:Hide()
		self.auctionUI.progressLabel:Show()
		self.auctionUI.progressLabel:SetText("posting..")
		self.auctionUI.progress:Show()
		self.auctionUI.progress:SetWidth(1)
		self.auctionUI.postButton:Disable()
		self.auctionUI.scanButton:Disable()
	
		self.mode = "posting"
		self.postActive = false
		self.PostingAborted = false
		self.PostingSplitActive = false
		self.PostingPostActive = false
		self.updateHandler:Show()
		Capitalist:OnPostIteration()
	end

	self:DebugExit("StartPost")
end

function Capitalist:GetProductsInInventory()
	self:DebugEntry("GetProductsInInventory")

	-- count the number of products in our inventory
	local numProducts = 0
	
	for bag = 0, 4 do
		for slot = 1, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (self.list[name]) then
					local _, count = GetContainerItemInfo(bag, slot)
					numProducts = numProducts + count
				end
			end
		end
	end
	
	return self:DebugExit("GetProductsInInventory", numProducts)
end

function Capitalist:PostCallback(accepted)
	self:DebugEntry("PostCallback")

	self:DebugMisc(accepted)
	self:DebugMisc(self.InPostIteration)
	self:DebugMisc(self.PostingPostActive)

	local texture, count, locked, quality, readable = GetContainerItemInfo(self.PostPendingBag, self.PostPendingSlot)
	self:DebugMisc(texture)
	self:DebugMisc(count)
	self:DebugMisc(locked)
	self:DebugMisc(quality)
	self:DebugMisc(readable)

	-- REVIEW: When switching to multi-post, this needs to be a count==(Old-ListCount) instead of a nil check
	if ((count == nil) or (not accepted)) then
		if (self.InPostIteration) then
			self:DebugMisc("deferring")
			self:DebugMisc(accepted)
			self.PostCompleteDeferredParam = accepted
		else
			if (accepted) then
				Capitalist:PostCompleted()
			else
				Capitalist:PostAborted()
			end
			Capitalist:PostAttemptDone(accepted)
		end
	end

	self:DebugExit("PostCallback")
end

function Capitalist:PostCompleted()
	self:DebugEntry("PostCompleted")

	local productsInAH = self.db.profile.storage[realm][character].AH

	self.list[self.PostPendingName].buyoutSelf = self.list[self.PostPendingName].bestPrice

	if (not productsInAH[self.PostPendingName]) then
		productsInAH[self.PostPendingName] = 1
	else
		productsInAH[self.PostPendingName] = productsInAH[self.PostPendingName] + 1
	end

	self:DebugExit("PostCompleted")
end

function Capitalist:PostAborted()
	self:DebugEntry("PostAborted")

	-- reverse what was done in Capitalist:Post
	ClickAuctionSellItemButton()
	PickupContainerItem(self.PostPendingBag, self.PostPendingSlot)
	self.PostPendingBag = bag
	self.PostPendingSlot = slot

	self:DebugExit("PostAborted")
end

function Capitalist:PostAttemptDone(accepted)
	self:DebugEntry("PostAttemptDone")

	self.PostPendingName = nil
	self.PostPendingBag = nil
	self.PostPendingSlot = nil
	self.PostingAborted = not accepted
	self.PostingPostActive = false

	self:DebugMisc(self.productsToPost)
	self:DebugMisc(self:GetProductsInInventory())
	local alreadyPosted = self.productsToPost - self:GetProductsInInventory()
	self:DebugMisc(alreadyPosted)

	local widthNew = 100 * alreadyPosted / self.productsToPost
	self:DebugMisc(widthNew)
	self.auctionUI.progress:SetWidth(widthNew)

	Capitalist:OnPostIteration()

	self:DebugExit("PostAttemptDone")
end

function Capitalist:SplitCallback()
	self:DebugEntry("SplitCallback")
	self:DebugMisc(self.InPostIteration)
	self:DebugMisc(self.PostingSplitActive)

	local texture, count, locked, quality, readable = GetContainerItemInfo(self.SplitPendingTgtBag, self.SplitPendingTgtSlot)
	self:DebugMisc(texture)
	self:DebugMisc(count)
	self:DebugMisc(locked)
	self:DebugMisc(quality)
	self:DebugMisc(readable)

	if (count == self.SplitPendingSize) then
		if (self.InPostIteration) then
			self:DebugMisc("deferring")
			self.SplitCompleteDeferred = true
		else
			Capitalist:SplitCompleted()
			Capitalist:SplitAttemptDone()
		end
	end

	self:DebugExit("SplitCallback")
end

function Capitalist:SplitCompleted()
	self:DebugEntry("SplitCompleted")
	self:DebugExit("SplitCompleted")
end

function Capitalist:SplitAttemptDone()
	self:DebugEntry("SplitAttemptDone")

	self.SplitPendingSrcBag = nil
	self.SplitPendingSrcSlot = nil
	self.SplitPendingSize = nil
	self.SplitPendingTgtBag = nil
	self.SplitPendingTgtSlot = nil
	self.PostingSplitActive = false
	Capitalist:OnPostIteration()

	self:DebugExit("SplitAttemptDone")
end

function Capitalist:OnUpdatePosting()
--	self:DebugEntry("OnUpdatePosting")
	if (self.elapsed > 1) then
		self.elapsed = 0
	end
--	self:DebugExit("OnUpdatePosting")
end

function Capitalist:OnPostIteration()
	self:DebugEntry("OnPostIteration")
	self.InPostIteration = true

	self:DebugMisc("123")
	self:DebugMisc(self.PostingAborted)
	self:DebugMisc(self.PostingPostActive)
	self:DebugMisc(self.PostingSplitActive)
	self:DebugMisc("456")

	if ((not self.PostingAborted) and
		(not self.PostingPostActive) and
		(not self.PostingSplitActive)) then

		self:DebugMisc("P")
		self.PostingPostActive = true
		if (not self:TryPost()) then
			self:DebugMisc("p")
			self.PostingPostActive = false

			self:DebugMisc("S")
			self.PostingSplitActive = true
			if (not self:TrySplit()) then
				self:DebugMisc("s")
				self.PostingSplitActive = false
			end
		end

		if ((not self.PostingPostActive) and (not self.PostingSplitActive)) then
			self:DebugMisc(self.PostingAborted)
		end
	end

	if (((not self.PostingPostActive) and (not self.PostingSplitActive)) or
		self.PostingAborted) then
		self:DebugMisc("Z")
		self.updateHandler:Hide()
		
		self.auctionUI.progress:Hide()
		self.auctionUI.progressLabel:Hide()
		self.auctionUI.lastScanLabel:Show()
		self.auctionUI.scanButton:Enable()
		self.auctionUI.postButton:Enable()
	end

	self.InPostIteration = false

	self:DebugMisc(self.PostCompleteDeferredParam)
	if (not self.PostCompleteDeferredParam == nil) then
		local accepted = self.PostCompleteDeferredParam
		self.PostCompleteDeferredParam = nil
		PostCallback(accepted)
	end

	self:DebugMisc(self.SplitCompleteDeferred)
	if (not self.SplitCompleteDeferred == nil) then
		self.SplitCompleteDeferred = nil
		SplitCallback()
	end

	self:DebugExit("OnPostIteration")
end

function Capitalist:TryPost()
	self:DebugEntry("TryPost")

	for bag = 0, 4 do
		self:DebugMisc(bag)
		for slot = 1, GetContainerNumSlots(bag) do
			self:DebugMisc(slot)
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				self:DebugMisc(link)
				local name = link:match("%[(.*)%]")
				self:DebugMisc(name)
				if (self.list[name] and self:ShouldPostThisItem(name)) then
					local texture, count, locked, quality, readable = GetContainerItemInfo(bag, slot)
					self:DebugMisc(texture)
					self:DebugMisc(count)
					self:DebugMisc(locked)
					self:DebugMisc(quality)
					self:DebugMisc(readable)
					if (not locked) then
						if (count == 1) then
							self:DebugMisc("A")
							self.PostPendingName = name
							self.PostPendingBag = bag
							self.PostPendingSlot = slot
							PickupContainerItem(bag, slot)
							self:DebugMisc("B")
							ClickAuctionSellItemButton()
							self:DebugMisc("C")

							-- preliminary bug check
							if (type(self.list[name].bestPrice) == "number") then
								self:DebugMisc("D")
								StartAuction(self.list[name].bestPrice, self.list[name].bestPrice, 3, 1, 1)

								self:DebugMisc("E")
								return self:DebugExit("TryPost", true)
							end
						end
					end
				end
			end
		end
	end

	return self:DebugExit("TryPost", false)
end

function Capitalist:TrySplit()
	self:DebugEntry("TrySplit")

	local emptySlot = nil
	local splitCandidate = nil	
	for bag = 0, 4 do
		local _, bagType = GetContainerNumFreeSlots(bag)
		if (bagType == 0) then
			for slot = 1, GetContainerNumSlots(bag) do
				if ((emptySlot == nil) or (splitCandidate == nil)) then
					local link = GetContainerItemLink(bag, slot)
					if (link == nil) then
						if (emptySlot == nil) then
							emptySlot = {bag, slot}
							self:DebugMisc("emptySlot found")
						end
					else
						if (splitCandidate == nil) then
							local name = link:match("%[(.*)%]")
							if (self.list[name] and self:ShouldPostThisItem(name)) then
								_, count, locked = GetContainerItemInfo(bag, slot)
								if (count > 1) then
									if (not locked) then
										splitCandidate = {bag, slot, 1}
										self:DebugMisc("splitCandidate found")
									end
								end
							end
						end
					end
				end
			end
		end
	end
	
	if ((not (emptySlot == nil)) and (not (splitCandidate == nil))) then
		self:DebugMisc(emptySlot)
		self:DebugMisc(splitCandidate)

		self.SplitPendingSrcBag = splitCandidate[1]
		self.SplitPendingSrcSlot = splitCandidate[2]
		self.SplitPendingSize = splitCandidate[3]
		self.SplitPendingTgtBag = emptySlot[1]
		self.SplitPendingTgtSlot = emptySlot[2]

		self:DebugMisc(splitCandidate[1])
		self:DebugMisc(splitCandidate[2])
		self:DebugMisc(splitCandidate[3])
		SplitContainerItem(splitCandidate[1], splitCandidate[2], splitCandidate[3])

		self:DebugMisc(emptySlot[1])
		self:DebugMisc(emptySlot[2])
		PickupContainerItem(emptySlot[1], emptySlot[2])

		self:DebugMisc("q")

		return self:DebugExit("TrySplit", true)
	else
		self.PostingSplitActive = false
		return self:DebugExit("TrySplit", false)
	end
end

function Capitalist:ShouldPostThisItem(name)
	self:DebugEntry("ShouldPostThisItem")
	self:DebugMisc(name)

	local productsInAH = self.db.profile.storage[realm][character].AH[name]
	self:DebugMisc(productsInAH)

	if (false) then
		local skillName = self.list[name].skillName
		self:DebugMisc(skillName)

		local product = self.db.profile.professions[profession][skillName]

		self:DebugMisc(product.count)

--	if (productsInAH and productsInAH >= product.count) then
		self:DebugMisc("-> false")
		return self:DebugExit("ShouldPostThisItem", false)
	end
	
	self:DebugMisc("-> true")

	return self:DebugExit("ShouldPostThisItem", true)
end
